# Filename: LifeCycleLabor.mod
# This program solves the life cycle problem of optimal consumption and saving with 
# labor income along the life.
#
# The life cycle problem:
# The utility for period t is util[t] = u(c[t]) - v(L[t]), where c[t] is the consumption, 
# L[t] is the labor supply, u is the utility function for consumption, v is the disutility 
# function for labor supply.
# If the saving at time t is S[t], then we have the budget constraints:
#    S[t+1] = (1+r)*S[t] + L[t+1]*w[t+1] - c[t+1]
# where w[t] is the wage.
# We do not allow negative consumption of any good. Furthermore, to avoid evaluating functions 
# like the log at 0, we impose a strict positivity condition:
# c[t]>=eps
# for some small positive number eps.
#
# Thus the life cycle problem is 
# maximize sum {t in 1..T} beta^t*(u(c[t]) - v(L[t]));
# subject to  S[t+1] = (1+r)*S[t] + L[t+1]*w[t+1] - c[t+1]; c>=eps; L>=0.0; S>=Smin; 


# PARAMETERS
# basics
param T integer := 45;      		# Number of time periods
param S0 := 0;        			# initial savings
param ST := 0;        			# final savings
param eps := 1e-3;    			# lower bound of consumption

# interest rate
param r := 0.06;              		# interest rate

# taste parameters
param beta := 0.96;           		# discount factor
param gamma := 2;     			# risk-aversion coefficient for consumption
param eta := 1;       			# labor supply parameter

# wage path parameters
param w{1..T};        			# wages
param R integer :=  35;      		# Age of maximum wage
      let {t in 1..(R-1)} w[t] := t/R;    
      let {t in R..T} w[t] := (t-T)/(R-T);    

# borrowing constraint
param Smin := 0;           			# lower bound of savings


# variables
var S{1..(T-1)} >= Smin;    		# savings -- this is a hard constraint
var c{1..T} >= eps;     			# consumptions
var L{1..T} >= 0;       			# labor supply
var util{t in 1..T} = c[t]^(1-gamma)/(1-gamma) - L[t]^(1+eta)/(1+eta); # utility function

# model: objective function and constraints
maximize Total_utility: sum {t in 1..T} beta^t*util[t];
subject to budget {t in 1..(T-2)}: S[t+1] = (1+r)*S[t] + L[t+1]*w[t+1] - c[t+1];
subject to budget0: S[1] = (1+r)*S0 + L[1]*w[1] - c[1];
subject to budgetT: ST = (1+r)*S[T-1] + L[T]*w[T] - c[T];

problem LifeCycle: Total_utility, budget, budget0, budgetT, S, c, L, util;
 
# initial guess
      let {t in 1..T} L[t] := 1;
      let {t in 1..(T-1)} c[t] := 0.5;
      let S[1] := (1+r)*S0 + L[1]*w[1] - c[1];
      let {t in 1..(T-2)} S[t+1] := (1+r)*S[t] + L[t+1]*w[t+1] - c[t+1];
      let c[T] := (1+r)*S[T-1] + L[T]*w[T] - ST;


# options for KNITRO
option knitro_options $knitro_options ' alg=0';    # algorithm/optimizer used: 
                                                   # 0: automatic algorithm selection
                                                   # 1: Interior/Direct algorithm
                                                   # 2: Interior/CG algorithm
                                                   # 3: Active algorithm 
option knitro_options $knitro_options ' opttol=1.0e-6';  # optimality termination tolerance (relative) 
option knitro_options $knitro_options ' feastol=1.0e-6'; # feasibility termination tolerance (relative)
option knitro_options $knitro_options ' maxit=10000'; # maximum number of major iterations  
option knitro_options $knitro_options ' outlev=0';   # printing output level: 
                                                     # 0: no printing 
                                                     # 1: print summary information
                                                     # 2: print information every 10 major iterations
                                                     # 3: print information at each major iteration
                                                     # 4: print information at each major and minor iteration
                                                     # 5: also print final (primal) variables
                                                     # 6: also print final constraint values and Lagrange multipliers

	

      solve;

      # display parameters
      display beta,T,R,Smin,r,w;

      # display solutions
      display c, L, S;

      # display shadow prices, bounds and slacks
      display {j1 in 1.._ncons} (_conname[j1],_con[j1].lb,_con[j1].body,_con[j1].ub,_con[j1].dual,_con[j1].slack);
      display {j1 in 1.._nvars} (_varname[j1],_var[j1].lb,_var[j1],_var[j1].ub,_var[j1].rc);

param income;			
	let income := sum {t in 1..T} L[t]*w[t]/((1+r)^t);
      
display Total_utility, income;



